#[derive(Clone, PartialEq, Debug)]
pub struct Node {
    pub kind: NodeKind,
    pub typ: Type,
    pub table_id: usize,
}

#[derive(Clone, PartialEq, Debug)]
pub enum NodeKind {
    Error,
    Id(u64),
    Int(i64),
    Str(String),
    Group(Box<Node>),
    Block(Vec<Node>),
    Call {
        subject: Box<Node>,
        args: Vec<Node>,
    },
    If {
        cond: Box<Node>,
        truthy: Box<Node>,
        falsy: Option<Box<Node>>,
    },
    Loop {
        body: Box<Node>,
    },
    Break,
    Assign {
        subject: Box<Node>,
        value: Box<Node>,
    },
    Let {
        subject: Box<Node>,
        value: Box<Node>,
    },
    Fn {
        subject: Box<Node>,
        params: Vec<Node>,
        return_typ: Type,
        body: Box<Node>,
        id: u64,
    },
    Return {
        value: Option<Box<Node>>,
    },
    Param {
        subject: Box<Node>,
        typ: Option<Type>,
    },
}

#[derive(Clone, PartialEq, Debug)]
pub enum Type {
    Error,
    Unit,
    I32,
    U32,
    Str,
    Fn {
        id: u64,
        params: Vec<Type>,
        return_typ: Box<Type>,
    },
}
