enum Duplicate<T: std::hash::Hash> {
    None(std::collections::HashMap<T, T>),
    Found(T),
}

pub trait Itertewls<T>
where
    Self: Iterator<Item = T> + Sized,
{
    fn find_first_duplicate(self) -> Option<T>;
}

impl<I, Item> Itertewls<Item> for I
where
    I: Iterator<Item = Item> + Sized,
    Item: std::cmp::PartialEq + Clone,
{
    fn find_first_duplicate(mut self) -> Option<Item> {
        self.try_fold(Vec::new(), |mut used, item| {
            if used.contains(&item) {
                Err(item)
            } else {
                used.push(item);
                Ok(used)
            }
        })
        .err()
    }
}
