#[derive(Clone, PartialEq, Debug)]
pub enum Node {
    Error,
    Id(u64),
    Int(i64),
    Str(String),
    Group(Box<Node>),
    Block(Vec<Node>),
    Call {
        subject: Box<Node>,
        args: Vec<Node>,
    },
    If {
        cond: Box<Node>,
        truthy: Box<Node>,
        falsy: Option<Box<Node>>,
    },
    Loop {
        body: Box<Node>,
    },
    Break,
    Assign {
        subject: Box<Node>,
        value: Box<Node>,
    },
    Let {
        subject: Box<Node>,
        value: Box<Node>,
    },
    Fn {
        subject: Box<Node>,
        params: Vec<Node>,
        return_typ: Box<Node>,
        body: Box<Node>,
    },
    Return {
        value: Option<Box<Node>>,
    },
    Param {
        subject: Box<Node>,
        typ: Option<Box<Node>>,
    },
}
